@@ -2,7 +2,7 @@
 #
 # SPDX-License-Identifier: MIT
 """
-`bmi160`
+`BMI160`
 ================================================================================
 
 Driver for the BMI160 sensor
@@ -20,6 +20,7 @@ Implementation Notes
 
 
 * Adafruit's Bus Device library: https://github.com/adafruit/Adafruit_CircuitPython_BusDevice
+
 * Adafruit's Register library: https://github.com/adafruit/Adafruit_CircuitPython_Register
 
 """
@@ -32,12 +33,11 @@ from adafruit_register.i2c_bits import RWBits
 
 try:
     from busio import I2C
-    from typing_extensions import NoReturn
     from typing import Tuple
 except ImportError:
     pass
 
-__version__ = "0.2.0"
+__version__ = "0.2.2"
 __repo__ = "https://github.com/jposada202020/CircuitPython_BMI160.git"
 
 
@@ -48,6 +48,7 @@ _COMMAND = const(0x7E)
 _ACCEL_CONFIG = const(0x40)
 _ACC_RANGE = const(0x41)
 _GYRO_CONFIG = const(0x42)
+_GYRO_RANGE = const(0x43)
 
 # RESET Command
 RESET_COMMAND = const(0xB6)
@@ -66,51 +67,93 @@ BANDWIDTH_400 = const(0b1010)  # 400 Hz
 BANDWIDTH_800 = const(0b1011)  # 800 Hz
 BANDWIDTH_1600 = const(0b1100)  # 1600 Hz
 BANDWIDTH_3200 = const(0b1101)  # 3200 Hz
+bandwidth_values = (
+    BANDWIDTH_25_32,
+    BANDWIDTH_25_16,
+    BANDWIDTH_25_8,
+    BANDWIDTH_25_4,
+    BANDWIDTH_25_2,
+    BANDWIDTH_25,
+    BANDWIDTH_50,
+    BANDWIDTH_100,
+    BANDWIDTH_200,
+    BANDWIDTH_400,
+    BANDWIDTH_800,
+    BANDWIDTH_1600,
+    BANDWIDTH_3200,
+)
+gyro_bandwidth_values = (
+    BANDWIDTH_25,
+    BANDWIDTH_50,
+    BANDWIDTH_100,
+    BANDWIDTH_200,
+    BANDWIDTH_400,
+    BANDWIDTH_800,
+    BANDWIDTH_1600,
+    BANDWIDTH_3200,
+)
 
 # Acceleration Range
 ACCEL_RANGE_2G = const(0b0011)
 ACCEL_RANGE_4G = const(0b0101)
 ACCEL_RANGE_8G = const(0b1000)
 ACCEL_RANGE_16G = const(0b1100)
+acc_range_values = (ACCEL_RANGE_2G, ACCEL_RANGE_4G, ACCEL_RANGE_8G, ACCEL_RANGE_16G)
 
 # UNDERSAMPLE
 NO_UNDERSAMPLE = const(0)
 UNDERSAMPLE = const(1)
+acc_sample_values = (NO_UNDERSAMPLE, UNDERSAMPLE)
 
-# Bandwith Parameter
+# Bandwidth Parameter
 FILTER = const(0)
 AVERAGING = const(1)
+acc_bandwidth_values = (FILTER, AVERAGING)
 
 # Acceleration Data
 ACC_X_LSB = const(0x12)
-ACC_X_MSB = const(0x13)
 ACC_Y_LSB = const(0x14)
-ACC_Y_MSB = const(0x15)
 ACC_Z_LSB = const(0x16)
-ACC_Z_MSB = const(0x17)
 
 # Acc Power Modes
 ACC_POWER_SUSPEND = const(0x10)
 ACC_POWER_NORMAL = const(0x11)
 ACC_POWER_LOWPOWER = const(0x12)
+acc_power_mode_values = (ACC_POWER_LOWPOWER, ACC_POWER_NORMAL, ACC_POWER_SUSPEND)
 
 # Temperature
 TEMP_LSB = const(0x20)
-TEMP_MSB = const(0x21)
 
-# Acceleration Data
+# Gyro Data
 GYRO_X_LSB = const(0x0C)
-GYRO_X_MSB = const(0x0D)
 GYRO_Y_LSB = const(0x0E)
-GYRO_Y_MSB = const(0x0F)
 GYRO_Z_LSB = const(0x10)
-GYRO_Z_MSB = const(0x11)
 
 # Gyro Cutoffs
 GYRO_NORMAL = const(0b10)
 GYRO_OSR2 = const(0b01)
 GYRO_OSR4 = const(0b00)
+gyro_cutoffs_values = (GYRO_OSR4, GYRO_OSR2, GYRO_NORMAL)
 
+# Gyro Power Modes
+GYRO_POWER_SUSPEND = const(0x14)
+GYRO_POWER_NORMAL = const(0x15)
+GYRO_POWER_FASTSTARTUP = const(0x17)
+gyro_power_modes = (GYRO_POWER_SUSPEND, GYRO_POWER_NORMAL, GYRO_POWER_FASTSTARTUP)
+
+# Gyro Ranges
+GYRO_RANGE_2000 = const(0b000)
+GYRO_RANGE_1000 = const(0b001)
+GYRO_RANGE_500 = const(0b010)
+GYRO_RANGE_250 = const(0b011)
+GYRO_RANGE_125 = const(0b100)
+gyro_values = (
+    GYRO_RANGE_125,
+    GYRO_RANGE_250,
+    GYRO_RANGE_500,
+    GYRO_RANGE_1000,
+    GYRO_RANGE_2000,
+)
 
 # pylint: disable= invalid-name, too-many-instance-attributes, missing-function-docstring
 
@@ -128,23 +171,24 @@ class BMI160:
     Here is an example of using the :class:`BMI160` class.
     First you will need to import the libraries to use the sensor
 
-        .. code-block:: python
+    .. code-block:: python
 
-            import board
-            import bmi160 as BMI160
+        import board
+        import bmi160 as BMI160
 
     Once this is done you can define your `board.I2C` object and define your sensor object
 
-        .. code-block:: python
+    .. code-block:: python
 
-            i2c = board.I2C()  # uses board.SCL and board.SDA
-            bmi = BMI160.BMI160(i2c)
+        i2c = board.I2C()  # uses board.SCL and board.SDA
+        bmi = BMI160.BMI160(i2c)
 
     Now you have access to the attributes
 
-        .. code-block:: python
+    .. code-block:: python
 
-            accx, accy, accz = bmi.acceleration
+        accx, accy, accz = bmi.acceleration
+        gyrox, gyroy, gyroz = bmi.gyro
 
 
     """
@@ -157,12 +201,9 @@ class BMI160:
     _gyro_config = UnaryStruct(_GYRO_CONFIG, "B")
 
     # Acceleration Data
-    _acc_data_x_msb = UnaryStruct(ACC_X_MSB, "B")
-    _acc_data_x_lsb = UnaryStruct(ACC_X_LSB, "B")
-    _acc_data_y_msb = UnaryStruct(ACC_Y_MSB, "B")
-    _acc_data_y_lsb = UnaryStruct(ACC_Y_LSB, "B")
-    _acc_data_z_msb = UnaryStruct(ACC_Z_MSB, "B")
-    _acc_data_z_lsb = UnaryStruct(ACC_Z_LSB, "B")
+    _acc_data_x = UnaryStruct(ACC_X_LSB, "<h")
+    _acc_data_y = UnaryStruct(ACC_Y_LSB, "<h")
+    _acc_data_z = UnaryStruct(ACC_Z_LSB, "<h")
     _read = UnaryStruct(_COMMAND, "B")
 
     # ACC_CONF Register (0x40)
@@ -175,25 +216,36 @@ class BMI160:
     # ACC_RANGE Register (0x41)
     # The register allows the selection of the accelerometer g-range
     _acc_range = RWBits(4, _ACC_RANGE, 0)
-    acceleration_scale = {3: 16384, 5: 8192, 8: 4096, 12: 2048}
+    acceleration_scale = {
+        "ACCEL_RANGE_2G": 16384,
+        "ACCEL_RANGE_4G": 8192,
+        "ACCEL_RANGE_8G": 4096,
+        "ACCEL_RANGE_16G": 2048,
+    }
+    gyro_scale = {
+        "GYRO_RANGE_125": 16.4,
+        "GYRO_RANGE_250": 32.8,
+        "GYRO_RANGE_500": 65.6,
+        "GYRO_RANGE_1000": 131.2,
+        "GYRO_RANGE_2000": 262.4,
+    }
 
     # Temperature
-    _temp_data_msb = UnaryStruct(TEMP_MSB, "B")
-    _temp_data_lsb = UnaryStruct(TEMP_LSB, "B")
+    _temp_data = UnaryStruct(TEMP_LSB, "<h")
 
     # Gyro Data
-    _gyro_data_x_msb = UnaryStruct(GYRO_X_MSB, "B")
-    _gyro_data_x_lsb = UnaryStruct(GYRO_X_LSB, "B")
-    _gyro_data_y_msb = UnaryStruct(GYRO_Y_MSB, "B")
-    _gyro_data_y_lsb = UnaryStruct(GYRO_Y_LSB, "B")
-    _gyro_data_z_msb = UnaryStruct(GYRO_Z_MSB, "B")
-    _gyro_data_z_lsb = UnaryStruct(GYRO_Z_LSB, "B")
+    _gyro_data_x = UnaryStruct(GYRO_X_LSB, "<h")
+    _gyro_data_y = UnaryStruct(GYRO_Y_LSB, "<h")
+    _gyro_data_z = UnaryStruct(GYRO_Z_LSB, "<h")
 
     # GYRO_CONF Register (0x41)
     # Sets the output data rate, the bandwidth, and the read mode of the gyro
     # sensor
-    _gyro_bwp = RWBits(2, _ACCEL_CONFIG, 4)
-    _gyro_odr = RWBits(4, _ACCEL_CONFIG, 0)
+    _gyro_bwp = RWBits(2, _GYRO_CONFIG, 4)
+    _gyro_odr = RWBits(4, _GYRO_CONFIG, 0)
+
+    # GYRO_RANGE Register (0x43)
+    _gyro_range = RWBits(3, _GYRO_RANGE, 0)
 
     def __init__(self, i2c_bus: I2C, address: int = _I2C_ADDR) -> None:
         self.i2c_device = i2c_device.I2CDevice(i2c_bus, address)
@@ -207,21 +259,20 @@ class BMI160:
         time.sleep(0.1)
         self._read = ACC_POWER_NORMAL
         time.sleep(0.1)
-        self._read = 0x15
+        self._read = GYRO_POWER_NORMAL
         time.sleep(0.1)
-        self._read = 0x19
 
-    def soft_reset(self) -> NoReturn:
+    def soft_reset(self) -> None:
         """
         Performs a Soft Reset
 
-        :return: NoReturn
+        :return: None
 
         """
         self._soft_reset = RESET_COMMAND
         time.sleep(0.015)
 
-    def error_code(self) -> NoReturn:
+    def error_code(self) -> None:
         """
         The register is meant for debug purposes, not for regular verification
         if an operation completed successfully.
@@ -254,7 +305,7 @@ class BMI160:
             print("Fatal Error")
 
     @property
-    def acceleration_undersample(self) -> int:
+    def acceleration_undersample(self) -> str:
         """
         The undersampling parameter is typically used in low power mode.
         When acc_us is set to ‘0’ and the accelerometer is in low-power mode,
@@ -271,14 +322,17 @@ class BMI160:
         +----------------------------------------+-------------------------+
 
         """
-        return self._acc_us
+        sample_values = ("NO_UNDERSAMPLE", "UNDERSAMPLE")
+        return sample_values[self._acc_us]
 
     @acceleration_undersample.setter
-    def acceleration_undersample(self, value: int) -> NoReturn:
+    def acceleration_undersample(self, value: int) -> None:
+        if value not in acc_sample_values:
+            raise ValueError("Value must be a valid acceleration undersample value")
         self._acc_us = value
 
     @property
-    def acceleration_bandwidth_parameter(self) -> int:
+    def acceleration_bandwidth_parameter(self) -> str:
         """
         Determines filter configuration (acc_us=0) and averaging for
         undersampling mode (acc_us=1).
@@ -292,14 +346,17 @@ class BMI160:
         +----------------------------------------+-------------------------+
 
         """
-        return self._acc_bwp
+        values = ("FILTER", "AVERAGING")
+        return values[self._acc_bwp]
 
     @acceleration_bandwidth_parameter.setter
-    def acceleration_bandwidth_parameter(self, value: int) -> NoReturn:
+    def acceleration_bandwidth_parameter(self, value: int) -> None:
+        if value not in acc_bandwidth_values:
+            raise ValueError("Value must a be a valid Acceleration bandwidth setting")
         self._acc_bwp = value
 
     @property
-    def acceleration_output_data_rate(self) -> int:
+    def acceleration_output_data_rate(self) -> str:
         """
         Define the output data rate in Hz is given by :math:`100/2^(8-accodr)`
         The output data rate is independent of the power mode setting for the sensor
@@ -338,14 +395,31 @@ class BMI160:
         +----------------------------------------+---------------------------------+
 
         """
-        return self._acc_odr
+        values = (
+            "BANDWIDTH_25_32",
+            "BANDWIDTH_25_16",
+            "BANDWIDTH_25_8",
+            "BANDWIDTH_25_4",
+            "BANDWIDTH_25_2",
+            "BANDWIDTH_25",
+            "BANDWIDTH_50",
+            "BANDWIDTH_100",
+            "BANDWIDTH_200",
+            "BANDWIDTH_400",
+            "BANDWIDTH_800",
+            "BANDWIDTH_1600",
+            "BANDWIDTH_3200",
+        )
+        return values[self._acc_odr]
 
     @acceleration_output_data_rate.setter
-    def acceleration_output_data_rate(self, value: int) -> NoReturn:
+    def acceleration_output_data_rate(self, value: int) -> None:
+        if value not in bandwidth_values:
+            raise ValueError("Value must be a valid Acceleration Data Rate setting")
         self._acc_odr = value
 
     @property
-    def acceleration_range(self) -> int:
+    def acceleration_range(self) -> str:
         """
         The register allows the selection of the accelerometer g-range.
         Changing the range of the accelerometer does not clear the data
@@ -366,10 +440,18 @@ class BMI160:
         +----------------------------------------+-------------------------+
 
         """
-        return self._acc_range
+        values = {
+            3: "ACCEL_RANGE_2G",
+            5: "ACCEL_RANGE_4G",
+            8: "ACCEL_RANGE_8G",
+            12: "ACCEL_RANGE_16G",
+        }
+        return values[self._acc_range]
 
     @acceleration_range.setter
-    def acceleration_range(self, value: int) -> NoReturn:
+    def acceleration_range(self, value: int) -> None:
+        if value not in acc_range_values:
+            raise ValueError("Value must be a valid Acceleration Range setting")
         self._acc_range = value
 
     @property
@@ -377,12 +459,12 @@ class BMI160:
 
         factor = self.acceleration_scale[self.acceleration_range]
 
-        x = (self._acc_data_x_msb * 256 + self._acc_data_x_lsb) / factor
-        y = (self._acc_data_y_msb * 256 + self._acc_data_y_lsb) / factor
-        z = (self._acc_data_z_msb * 256 + self._acc_data_z_lsb) / factor
+        x = self._acc_data_x / factor
+        y = self._acc_data_y / factor
+        z = self._acc_data_z / factor
         return x, y, z
 
-    def power_mode_status(self) -> NoReturn:
+    def power_mode_status(self) -> None:
         values = self._power_mode
 
         acc_pmu_status = (values & 0x18) >> 4
@@ -397,7 +479,7 @@ class BMI160:
         print("Gyro Power Mode", gyr_pmu_codes[gyr_pmu_status])
         print("Mag Power Mode", mag_pmu_codes[mag_pmu_status])
 
-    def acc_power_mode(self, value: int) -> NoReturn:
+    def acc_power_mode(self, value: int) -> None:
         """
         +----------------------------------------+-------------------------+
         | Mode                                   | Value                   |
@@ -410,6 +492,8 @@ class BMI160:
         +----------------------------------------+-------------------------+
 
         """
+        if value not in acc_power_mode_values:
+            raise ValueError("Value must be a valid Acceleration Power Mode Setting")
         self._read = value
         time.sleep(0.1)
 
@@ -426,20 +510,20 @@ class BMI160:
         :return: int
         """
 
-        return ((self._temp_data_msb * 256 + self._temp_data_lsb) * 1 / 2**9) + 23
+        return (self._temp_data * 1 / 2**9) + 23
 
     @property
     def gyro(self) -> Tuple[int, int, int]:
 
-        factor = 16.4
+        factor = self.gyro_scale[self.gyro_range]
 
-        x = (self._gyro_data_x_msb * 256 + self._gyro_data_x_lsb) / factor
-        y = (self._gyro_data_y_msb * 256 + self._gyro_data_y_lsb) / factor
-        z = (self._gyro_data_z_msb * 256 + self._gyro_data_z_lsb) / factor
+        x = self._gyro_data_x / factor
+        y = self._gyro_data_y / factor
+        z = self._gyro_data_z / factor
         return x, y, z
 
     @property
-    def gyro_output_data_rate(self) -> int:
+    def gyro_output_data_rate(self) -> str:
         """
         Define the output data rate in Hz is given by :math:`100/2^(8-gyroodr)`
         The output data rate is independent of the power mode setting for the sensor
@@ -447,7 +531,7 @@ class BMI160:
         Configurations without a bandwidth number are illegal settings and will
         result in an error code in the Register (0x02) ERR_REG.
 
-        ..warning ::
+        .. warning ::
             Lower ODR values than 25Hz are not allowed. If they are used they result
             in an error code in Register (0x02) ERR_REG.
 
@@ -474,14 +558,27 @@ class BMI160:
         +----------------------------------------+---------------------------------+
 
         """
-        return self._gyro_odr
+        values = (
+            "BANDWIDTH_25",
+            "BANDWIDTH_50",
+            "BANDWIDTH_100",
+            "BANDWIDTH_200",
+            "BANDWIDTH_400",
+            "BANDWIDTH_800",
+            "BANDWIDTH_1600",
+            "BANDWIDTH_3200",
+        )
+
+        return values[self._gyro_odr]
 
     @gyro_output_data_rate.setter
-    def gyro_output_data_rate(self, value: int) -> NoReturn:
+    def gyro_output_data_rate(self, value: int) -> None:
+        if value not in gyro_bandwidth_values:
+            raise ValueError("Value must be a valid Gyro Data Rate setting")
         self._gyro_odr = value
 
     @property
-    def gyro_bandwidth_parameter(self) -> int:
+    def gyro_bandwidth_parameter(self) -> str:
         """
         The gyroscope bandwidth coefficient defines the 3 dB cutoff frequency
          of the low pass filter for the sensor data.
@@ -521,8 +618,81 @@ class BMI160:
         +----------------------------------------+-------------------------+
 
         """
-        return self._gyro_bwp
+        values = ("GYRO_OSR4", "GYRO_OSR2", "GYRO_NORMAL")
+        return values[self._gyro_bwp]
 
     @gyro_bandwidth_parameter.setter
-    def gyro_bandwidth_parameter(self, value: int) -> NoReturn:
+    def gyro_bandwidth_parameter(self, value: int) -> None:
+        if value not in gyro_cutoffs_values:
+            raise ValueError("Value must be a valid Gyro Bandwidth setting")
         self._gyro_bwp = value
+
+    @property
+    def gyro_power_mode(self) -> str:
+        """
+        +-------------------------------------------+-------------------------+
+        | Mode                                      | Value                   |
+        +===========================================+=========================+
+        | :py:const:`BMI160.GYRO_POWER_SUSPEND`     | :py:const:`0x14`        |
+        +-------------------------------------------+-------------------------+
+        | :py:const:`BMI160.GYRO_POWER_NORMAL`      | :py:const:`0x15`        |
+        +-------------------------------------------+-------------------------+
+        | :py:const:`BMI160.GYRO_POWER_FASTSTARTUP` | :py:const:`0x17`        |
+        +-------------------------------------------+-------------------------+
+
+        """
+        g_power_modes = {
+            0x14: "GYRO_POWER_SUSPEND",
+            0x15: "GYRO_POWER_NORMAL",
+            0x17: "GYRO_POWER_FASTSTARTUP",
+        }
+
+        return g_power_modes[self._read]
+
+    @gyro_power_mode.setter
+    def gyro_power_mode(self, value: int) -> None:
+        if value not in gyro_power_modes:
+            raise ValueError("Value must be a valid Gyro Power Mode")
+
+        self._read = value
+        time.sleep(0.1)
+
+    @property
+    def gyro_range(self) -> str:
+        """
+        The register allows the selection of the gyro g-range.
+        Changing the range of the accelerometer does not clear the data
+        ready bit in the Register (0x1B) STATUS. It is recommended to
+        read the Register (0x04-0x17) DATA after the range change to
+        remove a stall data ready bit from before the range change.
+
+        +----------------------------------------+-------------------------+
+        | Mode                                   | Value                   |
+        +========================================+=========================+
+        | :py:const:`BMI160.GYRO_RANGE_2000`     | :py:const:`0b000`       |
+        +----------------------------------------+-------------------------+
+        | :py:const:`BMI160.GYRO_RANGE_1000`     | :py:const:`0b001`       |
+        +----------------------------------------+-------------------------+
+        | :py:const:`BMI160.GYRO_RANGE_500`      | :py:const:`0b010`       |
+        +----------------------------------------+-------------------------+
+        | :py:const:`BMI160.GYRO_RANGE_250`      | :py:const:`0b011`       |
+        +----------------------------------------+-------------------------+
+        | :py:const:`BMI160.GYRO_RANGE_125`      | :py:const:`0b100`       |
+        +----------------------------------------+-------------------------+
+
+        """
+        g_values = (
+            "GYRO_RANGE_125",
+            "GYRO_RANGE_250",
+            "GYRO_RANGE_500",
+            "GYRO_RANGE_1000",
+            "GYRO_RANGE_2000",
+        )
+
+        return g_values[self._gyro_range]
+
+    @gyro_range.setter
+    def gyro_range(self, value: int) -> None:
+        if value not in gyro_values:
+            raise ValueError("Value must be a valid Gyro range")
+        self._gyro_range = value
